# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys

from qommon.ctl import Command, make_option

class CmdConvertToSql(Command):
    name = 'convert-to-sql'

    def __init__(self):
        Command.__init__(self, [
                make_option('--dbname', metavar='DATABASE', action='store',
                            dest='dbname'),
                make_option('--user', metavar='USER', action='store',
                            dest='user'),
                make_option('--password', metavar='PASSWORD', action='store',
                            dest='password'),
                make_option('--host', metavar='HOSTNAME', action='store',
                            dest='host'),
                make_option('--port', metavar='PORT', action='store',
                            dest='port')
                ])

    def execute(self, base_options, sub_options, args):
        import publisher

        self.config.remove_option('main', 'error_log')
        publisher.WcsPublisher.configure(self.config, sub_options.extra)
        pub = publisher.WcsPublisher.create_publisher()

        hostname = args[0]
        pub.app_dir = os.path.join(pub.app_dir, hostname)
        pub.set_config()

        from wcs.formdef import FormDef
        from wcs import sql

        pub.cfg['postgresql'] = {
                'database': sub_options.dbname,
                'user': sub_options.user,
                'password': sub_options.password,
                'host': sub_options.host,
                'port': sub_options.port,
        }

        sql.get_connection_and_cursor(new=True)

        print 'converting users'
        from users import User
        sql.do_user_table()
        for user_id in User.keys():
            user = User.get(user_id)
            user.__class__ = sql.SqlUser
            user.store()
        sql.SqlUser.fix_sequences()

        for formdef in FormDef.select():
            print 'converting', formdef.name
            sql.do_formdef_tables(formdef)
            data_class = formdef.data_class(mode='files')
            delattr(sys.modules['formdef'], formdef.url_name.title())
            sql_data_class = formdef.data_class(mode='sql')
            for id in data_class.keys():
                formdata = data_class.get(id)
                formdata._formdef = formdef
                formdata._evolution = formdata.evolution
                formdata.__class__ = sql_data_class
                formdata.store()
                sql_data_class.store(formdata)
                sql_data_class.fix_sequences()


CmdConvertToSql.register()
