# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from quixote import redirect
from quixote.directory import Directory
from quixote.html import TemplateIO, htmltext

from qommon.form import *
from qommon.admin.menu import html_top, command_icon

try:
    from qommon.certificate import CertificateAuthorities
    from M2Crypto import X509
    m2crypto = True
except ImportError:
    m2crypto = False

class CertificatePage(Directory):
    _q_exports = ['', 'delete']

    def __init__(self, component):
        self.certificate = CertificateAuthorities.get(component)

    def _q_index(self):
        html_top('settings', title = _('Certificate %s') % self.certificate.id)
        r = TemplateIO(html=True)
        r += htmltext('<h2>%s</h2>') % (_('Certificate %s' % self.certificate.id))
        r += htmltext('<div>%s</div>') % (self.certificate.cert)
        return r.getvalue()

    def delete(self):
        form = Form(enctype='multipart/form-data')
        form.widgets.append(HtmlWidget('<p>%s</p>' % _(
                        "You are about to delete this certificate.")))
        form.add_submit("submit", _("Submit"))
        form.add_submit("cancel", _("Cancel"))
        if form.get_widget('cancel').parse():
            return redirect('..')
        if not form.is_submitted() or form.has_errors():
            get_response().breadcrumb.append(('delete', _('Delete')))
            html_top('settings', title = _('certificates'))
            return form.render()
        else:
            self.certificate.remove_self()
            return redirect('..')

class CertificatesDirectory(Directory):
    _q_exports = ['', 'add']

    def index(self):
        get_response().breadcrumb.append( ('certificates/', _('Certificates')) )
        html_top('settings', title = _('certificates'))
        r = TemplateIO(html=True)
        r += htmltext('<h2>%s</h2>') % _('Certificate Authorities')

        r += htmltext('<ul id="nav-forms-admin">')
        r += htmltext('  <li><a href="add">%s</a></li>') % _('Add Certificate Authority')
        r += htmltext('</ul>')

        r += htmltext('<ul class="biglist">')
        for ca in CertificateAuthorities.select(lambda x: x.ca == True):
            r += htmltext('<li>')
            r += htmltext('<strong class="label">%s</strong>') % ca.issuer
            r += htmltext('<p class="commands">')
            r += command_icon('%s/delete' % ca.id, 'remove', popup = True)
            r += htmltext('</p></li>')
        r += htmltext('</ul>')
        return r.getvalue()

    def _q_index(self):
        if not m2crypto:
            get_response().breadcrumb.append( ('certificates/', _('Certificates')) )
            html_top('settings', title = _('certificates'))
            return htmltext('<div class=errornotice>%s</div>') % \
                    _("You need to install M2Crypto to use this feature")
        else:
            return self.index()

    def add(self):
        get_response().breadcrumb.append( ('certificates/', _('Certificates')) )
        get_response().breadcrumb.append( ('add', _('Add')) )
        html_top('settings', title = _('Add Certificate Authority'))
        form = Form(enctype='multipart/form-data')
        form.add(FileWidget, 'ca', title = _('Certificate'), required = True)
        form.add_submit('submit', _('Submit'))
        form.add_submit('cancel', _('Cancel'))
        if form.get_widget('cancel').parse():
            return redirect('.')
        if not form.is_submitted() or form.has_errors():
            return form.render()
        else:
            f = form.get_widget('ca').parse().fp
            ca_str = f.read()
            f.close()
            try:
                c = CertificateAuthorities(ca_str)
                c.ca = True
                c.store()
                return redirect('.')
            except X509.X509Error:
                form.set_error("ca", _("Bad certificate"))
                return form.render()

    def _q_lookup(self, component):
        get_response().breadcrumb.append( ('certificates/', _('Certificates')) )
        try:
            return CertificatePage(component)
        except KeyError:
            raise errors.TraversalError()
