# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import urllib
import httplib

class SOAPException(Exception):
    def __init__(self, msg, err = None, url = None, body = None):
        """SOAP call failure, this is not related to SOAP Fault, which
           are normal SOAP response"""
        self.err = err
        self.url = url
        self.body = body
        super(SOAPException, self).__init__(msg)

    def __str__(self):
        return '<SOAPException url:%s sub-err:%s>' % (self.url, self.err)


def soapCall(url, msg, key = None, cert = None):
    """Use httplib to make a SOAP call, the content of the SOAP message must
       be passed in @msg and the destination in @url.
    """

    if url.startswith('http://'):
        host, query = urllib.splithost(url[5:])
        conn = httplib.HTTPConnection(host)
    else:
        host, query = urllib.splithost(url[6:])
        conn = httplib.HTTPSConnection(host,
                key_file = key or cert, cert_file = cert)
    try:
        conn.request('POST', query, msg, {'Content-Type': 'text/xml'})
        response = conn.getresponse()
    except Exception, err:
        # exception could be raised by request
        raise SOAPException("Connection failed",
                url = url, err = err, body = msg)
    data = response.read()
    conn.close()
    if response.status not in (200, 204): # 204 ok for federation termination
        raise SOAPException("Response status is not OK", url = url,
                body = msg)
    return data
